﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Common.Unit
{
    /// <summary>
    /// 构建动态表达式
    /// </summary>
    /// <typeparam name="Dto">表达式的类</typeparam>
    public class LamadaExtention<Dto> where Dto : new()
    {
        private List<Expression> m_lstExpression = null;
        private ParameterExpression m_Parameter = null;
        private List<Expression<Func<Dto, bool>>> otherList = null;

        /// <summary>构建动态表达式</summary>
        public LamadaExtention()
        {
            m_lstExpression = new List<Expression>();
            m_Parameter = Expression.Parameter(typeof(Dto), "x");
            otherList = new List<Expression<Func<Dto, bool>>>();
        }

        /// <summary>
        /// 初始化构建对象数据的Lambda查询
        /// </summary>
        /// <param name="T">需要初始化lambda的对象</param>
        //public LamadaExtention(Dto t)
        //{
        //    m_lstExpression = new List<Expression>();
        //    m_Parameter = Expression.Parameter(typeof(Dto), "x");
        //    otherList = new List<Expression<Func<Dto, bool>>>();
        //    if (t == null) return;
        //    PropertyInfo[] properties = t.GetType().GetProperties(BindingFlags.Instance | BindingFlags.Public);
        //    if (properties.Length <= 0) { return; }
        //    foreach (PropertyInfo item in properties)
        //    {
        //        string name = item.Name;
        //        object value = item.GetValue(t, null);
        //        if (value != null)
        //        { And(name, value); }
        //    }
        //}

        /// <summary>
        /// 构造表达式，这里是 and 的写法
        /// </summary>
        /// <param name="strPropertyName">字段</param>
        /// <param name="strValue">字段值</param>
        /// <param name="expressType">表达式</param>
        //public void And(string strPropertyName, object strValue, ExpressionType expressType = ExpressionType.Equal)
        //{
        //    Expression expRes = null;
        //    MemberExpression member = Expression.PropertyOrField(m_Parameter, strPropertyName);
        //    if (expressType == ExpressionType.Contains)
        //    {
        //        expRes = Expression.Call(member, typeof(string).GetMethod("Contains"), Expression.Constant(strValue));
        //    }
        //    else if (expressType == ExpressionType.Equal)
        //    {
        //        expRes = Expression.Equal(member, Expression.Constant(strValue, member.Type));
        //    }
        //    else if (expressType == ExpressionType.LessThan)
        //    {
        //        expRes = Expression.LessThan(member, Expression.Constant(strValue, member.Type));
        //    }
        //    else if (expressType == ExpressionType.LessThanOrEqual)
        //    {
        //        expRes = Expression.LessThanOrEqual(member, Expression.Constant(strValue, member.Type));
        //    }
        //    else if (expressType == ExpressionType.GreaterThan)
        //    {
        //        expRes = Expression.GreaterThan(member, Expression.Constant(strValue, member.Type));
        //    }
        //    else if (expressType == ExpressionType.GreaterThanOrEqual)
        //    {
        //        expRes = Expression.GreaterThanOrEqual(member, Expression.Constant(strValue, member.Type));
        //    }
        //    //return expRes;
        //    m_lstExpression.Add(expRes);
        //}

        /// <summary>
        /// 自定义的查询
        /// </summary>
        /// <typeparam name="Dto">当前的传入模型</typeparam>
        /// <param name="expLeft">左边的表达式</param>
        /// <param name="expRight">右边的表达式</param>
        public void And(Expression<Func<Dto, bool>> expRight)
        {
            otherList.Add(expRight);
            //  var replaceObj = new LamadaReplacer(m_Parameter);
            ////  Expression left = replaceObj.Replace((x=>true));
            //  Expression right = replaceObj.Replace(expRight.Body);
            //  BinaryExpression body = Expression.And(null, right);
            //  m_lstExpression.Add(body);
        }



        /// <summary>
        /// 构建针对Or条件的表达式
        /// </summary>
        /// <param name="strPropertyName">字段</param>
        /// <param name="lstValue"></param>
        public void Or(string strPropertyName, List<object> lstValue)
        {
            Expression expRes = null;
            MemberExpression member = Expression.PropertyOrField(m_Parameter, strPropertyName);
            foreach (var oValue in lstValue)
            {
                if (expRes == null)
                {
                    expRes = Expression.Equal(member, Expression.Constant(oValue, member.Type));
                }
                else
                {
                    expRes = Expression.Or(expRes, Expression.Equal(member, Expression.Constant(oValue, member.Type)));
                }
            }
            m_lstExpression.Add(expRes);
        }

        /// <summary>
        /// 得到Lamada表达式的Expression对象
        /// </summary>
        /// <returns>需要的表达式</returns>
        public Expression<Func<Dto, bool>> GetLambda()
        {
            Expression whereExpr = null;
            foreach (var expr in this.m_lstExpression)
            {
                if (whereExpr == null) whereExpr = expr;
                else whereExpr = Expression.And(whereExpr, expr);
            }
            var replaceObj = new LamadaReplacer(m_Parameter);
            foreach (var one in otherList)
            {
                Expression need = replaceObj.Replace(one.Body);
                if (whereExpr == null)
                {
                    whereExpr = need;
                }
                else
                {
                    whereExpr = Expression.And(whereExpr, need);
                }
            }
            if (whereExpr == null)
                return (x => true);
            return Expression.Lambda<Func<Dto, bool>>(whereExpr, m_Parameter);
        }
    }
    /// <summary>
    /// 用于区分操作的枚举
    /// </summary>
    public enum ExpressionType
    {
        /// <summary>包含 =like</summary>
        Contains,//like
        /// <summary>等于</summary>
        Equal,//等于
        /// <summary>小于</summary>
        LessThan,//小于
        /// <summary>小于等于</summary>
        LessThanOrEqual,//小于等于
        /// <summary>大于</summary>
        GreaterThan,//大于
        /// <summary>大于等于</summary>
        GreaterThanOrEqual//大于等于
    }
}