﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;
using Common.Unit.Redis;
using G.MES.DataModel;
using G.MES.ViewModel;
using Newtonsoft.Json;
using G.Buss.System;
namespace G.MES.API.App_Start
{
    /// <summary>
    /// 
    /// </summary>
    public class OperateTrackAttribute : ActionFilterAttribute
    {
        /// <summary>
        /// 
        /// </summary>
        private Sys_AuditLogSvc alsv = new Sys_AuditLogSvc();
        /// <summary>
        /// 自定义参数
        /// </summary>
        public string msg { get; set; }
        /// <summary>
        /// 请求参数
        /// </summary>
        private string RequestParam { get; set; }
        /// <summary>
        /// 
        /// </summary>
        public LoginInfo loginInfo { get; set; }
        /// <summary>
        /// 
        /// </summary>
        public OperateTrackAttribute()
        {

        }
        private static string sqlConn = ConfigurationManager.ConnectionStrings["MainDB"].ToString();
        /// <summary>
        /// 初始化时填入类的说明
        /// </summary>
        /// <param name="message"></param>
        public OperateTrackAttribute(string message)
        {
            msg = message;
        }


        private static readonly string key = "enterTime";

        /// <summary>
        /// 在每个请求上都会调用的方法
        /// </summary>
        /// <param name="actionContext"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override Task OnActionExecutingAsync(HttpActionContext actionContext,
                                                        CancellationToken cancellationToken)
        {
            try
            {
                loginInfo = GetLoginUser(actionContext);

                if (SkipLogging(actionContext))
                {
                    return base.OnActionExecutingAsync(actionContext, cancellationToken);
                }
                //记录进入请求的时间
                actionContext.Request.Properties[key] = DateTime.Now.ToBinary();//转64位二进制
                return base.OnActionExecutingAsync(actionContext, cancellationToken);
            }
            catch
            {
                return base.OnActionExecutingAsync(actionContext, cancellationToken);
            }

        }
        /// <summary>
        /// 请求完后执行方法，获取请求数据及返回数据
        /// </summary>
        /// <param name="actionExecutedContext"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override Task OnActionExecutedAsync(HttpActionExecutedContext actionExecutedContext,
                                                        CancellationToken cancellationToken)
        {
            try
            {
                if (loginInfo == null)
                {
                    loginInfo = new LoginInfo();
                    loginInfo.Sys_UserOID = new Guid();
                }
                object beginTime = null;
                if (actionExecutedContext.Request.Properties.TryGetValue(key, out beginTime))
                {
                    DateTime time = DateTime.FromBinary(Convert.ToInt64(beginTime));
                    HttpRequest request = HttpContext.Current.Request;
                    string token = "";
                    if (request.Params["api_key"] == null)
                    {
                        if (request.Headers["api_key"] == null)
                        {
                            return base.OnActionExecutedAsync(actionExecutedContext, cancellationToken);
                        }
                        else
                        {
                            token = request.Headers["api_key"];
                        }

                    }
                    else
                    {
                        token = request.Params["api_key"];
                    }
                    SystemLogging logging = new SystemLogging
                    {
                        SysytemLoggingOID = Guid.NewGuid(),
                        EnterTime = time,
                        EndTime = DateTime.Now,
                        CostTime = Convert.ToInt32((DateTime.Now - time).TotalMilliseconds),
                        RequestUri = request.Url.AbsoluteUri,
                        RequestParamaters = RequestParam,//GetRequestValues(actionExecutedContext),
                        ResponseParamaters = GetResponseValues(actionExecutedContext),
                        UserIP = request.UserHostName,
                        ServerIP = request.Browser.Browser +
                            "-" + request.Browser.Version + "-" + request.Browser.Type,
                        Browser = request.Browser.Browser,
                        Sys_User_FK = loginInfo.Sys_UserOID
                    };
                    Sys_AuditLogSvc logSvc = new Sys_AuditLogSvc();
                    logSvc.AddSysTemLogging(logging);

                }
                return base.OnActionExecutedAsync(actionExecutedContext, cancellationToken);
            }
            catch (Exception ex)
            {
                HttpRequest request = HttpContext.Current.Request;
                SystemLogging logging = new SystemLogging
                {
                    SysytemLoggingOID = Guid.NewGuid(),
                    EnterTime = DateTime.Now,
                    EndTime = DateTime.Now,
                    CostTime = 0,
                    RequestUri = request.Url.AbsoluteUri,
                    RequestParamaters = RequestParam,//GetRequestValues(actionExecutedContext),
                    ResponseParamaters = "系统日志后" + ex.Message,
                    UserIP = request.UserHostName,
                    ServerIP = request.Browser.Browser +
                    "-" + request.Browser.Version + "-" + request.Browser.Type,
                    Browser = request.Browser.Browser,
                    Sys_User_FK = loginInfo.Sys_UserOID
                };
                Sys_AuditLogSvc logSvc = new Sys_AuditLogSvc();
                logSvc.AddSysTemLogging(logging);
                return base.OnActionExecutedAsync(actionExecutedContext, cancellationToken);
            }

        }



        /// <summary>
        /// 判断是否需要记录
        /// </summary>
        /// <param name="actionContext"></param>
        /// <returns></returns>
        public static bool SkipLogging(HttpActionContext actionContext)
        {
            return actionContext.ActionDescriptor.GetCustomAttributes<NoLogAttribute>()
                .Any()
                || actionContext.ActionDescriptor.ControllerDescriptor
                .GetCustomAttributes<NoLogAttribute>().Any();
        }
        /// <summary>
        /// 获取request提交内容
        /// </summary>
        /// <param name="actionExecutedContext"></param>
        /// <returns></returns>
        public string GetRequestValues(HttpActionExecutedContext actionExecutedContext)
        {
            Stream stream = actionExecutedContext.Request.Content.ReadAsStreamAsync().Result;
            Encoding encoding = Encoding.UTF8;
            stream.Position = 0;
            var reader = new StreamReader(stream, encoding);
            string result = reader.ReadToEnd();

            //stream 下标指定到初始位，好方便之后的读取
            stream.Position = 0;
            return result;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="actionExecutedContext"></param>
        /// <returns></returns>
        public string GetResponseValues(HttpActionExecutedContext actionExecutedContext)
        {
            Stream stream = actionExecutedContext.Response.Content.ReadAsStreamAsync().Result;
            Encoding encoding = Encoding.UTF8;
            var reader = new StreamReader(stream, encoding);
            string result = reader.ReadToEnd();
            stream.Position = 0;
            return result;

        }
        private LoginInfo GetLoginUser(HttpActionContext actionContext)
        {
            if (actionContext.RequestContext.RouteData.Values["LoginUid"] == null)
            {
                return null;
            }
            string uid = actionContext.RequestContext.RouteData.Values["LoginUid"].ToString();
            RequestParam = JsonConvert.SerializeObject(actionContext.ActionArguments.Values);
            LoginInfo user = null;
            try
            {
                RedisHelper reids = new RedisHelper();
                user = reids.StringGet<LoginInfo>(uid);
            }
            catch { }
            return user;
        }

    }
}
