﻿using G.Buss.BaseInfo.Services.ClientInfoSVC;
using G.Buss.BaseInfo.ViewModel.ClientInfoModel;
using G.Buss.BaseInfo.ViewModel.PMIDModel;
using G.MES.API.App_Start;
using G.MES.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Http;

namespace G.MES.API.Controllers
{
    /// <summary>
    /// 受检者信息
    /// </summary>
    public class ClientInfoController : BasicController
    {
        public readonly ClientInfoSVC ClientInfo = new ClientInfoSVC();
        /// <summary>
        /// 获取受试者信息列表
        /// </summary>
        /// <param name="requestModel"></param>
        /// <returns></returns>
        [HttpPost]
        public PageData GetClientInfo(ClientInfoRequestModel requestModel)
        {
            try
            {
                LoginInfo user = GetLoginUser();
                return ClientInfo.DoGetClientInfo(requestModel, user);
            }
            catch (Exception ex)
            {
                throw new Exception("获取受试者信息表格" + ex.Message);
            }
        }

        /// <summary>
        /// 获取所有受试者信息列表
        /// </summary>
        /// <param name="requestModel"></param>
        /// <returns></returns>
        [HttpPost]
        public PageData GetAllClientInfo(ClientInfoRequestModel requestModel)
        {
            try
            {
                return ClientInfo.DoGetAllClientInfo(requestModel);
            }
            catch (Exception ex)
            {
                throw new Exception("获取受试者信息表格" + ex.Message);
            }
        }

        /// <summary>
        /// 获取详情
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public ClientInfoDetailModel GetClientInfoDetail(Guid ClientInfoOID)
        {
            try
            {
                List<Guid> ClientInfosOID = new List<Guid>() { ClientInfoOID };
                return ClientInfo.DoGetClientInfoDetail(ClientInfosOID).FirstOrDefault();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

        }

        /// <summary>
        /// 保存（新增或更新）
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public bool SaveClientInfo(ClientInfoSaveRequestModel requestModel)
        {
            try
            {
                LoginInfo user = GetLoginUser();
                ClientInfo.DoSaveClientInfo(requestModel, user);
                return true;
            }
            catch (Exception ex)
            {
                throw new Exception("保存受试者信息表格：" + ex.Message);
            }
        }
        /// <summary>
        /// 批量删除 
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public bool DelClientInfo(List<Guid> requestModel)
        {
            try
            {
                return ClientInfo.DoDelClientInfo(requestModel);
            }
            catch (Exception ex)
            {
                throw new Exception("删除受试者信息表格：" + ex.Message);
            }
        }

        /// <summary>
        /// excel导入
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public bool ImportClientInfo()
        {
            try
            {
                LoginInfo user = GetLoginUser();
                HttpPostedFile file = HttpContext.Current.Request.Files["Excel"];
                return ClientInfo.DoImportClientInfo(file, user);

            }
            catch (Exception ex)
            {
                throw new Exception("导入受试者信息表格：" + ex.Message);
            }

        }
        /// <summary>
        /// Excel导出
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public string ExportClient()
        {
            try
            {
                LoginInfo user = GetLoginUser(); 
                return ClientInfo.DoExportClientInfo(user);

            }
            catch (Exception ex)
            {
                throw new Exception("导入受试者信息表格：" + ex.Message);
            }
        }
    }
}