﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.ClientInfoModel;
using G.Buss.BaseInfo.ViewModel.CodeTableModel;
using G.Buss.BaseInfo.ViewModel.GeneCorrelationModel;
using G.Buss.BaseInfo.ViewModel.GeneticRisksModel;
using G.Buss.BaseInfo.ViewModel.MISReportModel;
using G.Buss.BaseInfo.ViewModel.SampleControlModel;
using G.Buss.BaseInfo.ViewModel.TMBGradeRequestModel;
using G.MES.DataModel;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace G.Buss.BaseInfo.Services.ExportWordSVC
{
    public class ReportDataSvc : BaseSvc
    {
        public readonly ClientInfoSVC.ClientInfoSVC ClientInfo = new ClientInfoSVC.ClientInfoSVC();

        /// <summary>
        /// 获取需要导出的受试者集合
        /// </summary>
        /// <param name="ClientInfosOID"></param>
        public ClientInfoDetailModel GetClirntInfo(Guid requestModel)
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                ClientInfoDetailModel thisQueryModel = context.Query<ClientInfo>().Where(a => requestModel == a.ClientInfoOID)
                    .Select(a => new ClientInfoDetailModel
                    {
                        ClientInfoOID = a.ClientInfoOID,
                        Name = a.Name,
                        Sex = a.Sex,
                        IDCard = a.IDCard,
                        Birth = a.Birth,
                        SampleCode = a.SampleCode,
                        SampleType = a.SampleType,
                        Phone = a.Phone,
                        SampleDetection = a.SampleDetection,
                        TechnologyDetection = a.TechnologyDetection,
                        TumorsType = a.TumorsType,
                        TNM = a.TNM,
                        IsEssential = a.IsEssential,
                        ClinicalDiagnosis = a.ClinicalDiagnosis,
                        FamilyHistory = a.FamilyHistory,
                        PersonalHistory = a.PersonalHistory,
                        SamplingTime = a.SamplingTime
                    }).FirstOrDefault();
                return thisQueryModel;
            }
        }

        /// <summary>
        /// 获取TMB等级说明
        /// </summary>
        public List<TMBTable> GetTMBLevel()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<TMBTable> TMBGradeQuery = context.JoinQuery<TMBGrade, Sys_Organize>((TMBGrade, org) => new object[] {
                        JoinType.LeftJoin,TMBGrade.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((TMBGrade, org) => new TMBTable
                    {
                        Grade = TMBGrade.Grade,
                        Scope = TMBGrade.Scope,
                        Remark = TMBGrade.Remark
                    }).ToList();
                return TMBGradeQuery;
            }
        }
        /// <summary>
        /// 获取MIS结果提示
        /// </summary>
        public List<MISReportQueryModel> GetMIS()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<MISReportQueryModel> MISReportQuery = context.JoinQuery<MISReport, Sys_Organize>((MISReport, org) => new object[] {
                        JoinType.LeftJoin,MISReport.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((MISReport, org) => new MISReportQueryModel
                    {
                        MISReportOID = MISReport.MISReportOID,
                        Grade = MISReport.Grade,
                        Scope = MISReport.Scope,
                        Remark = MISReport.Remark,
                        ClassCode = org.ClassCode
                    }).ToList();
                int TotalNum = MISReportQuery.Count();
                return MISReportQuery;
            }
        }
        /// <summary>
        /// 获取检测结果
        /// </summary>
        public DataTable GetCheckResult()
        {
            SqlHelper context = new SqlHelper();
            return context.ExcuteDatatable("SELECT * FROM dbo.Summary ORDER BY CurrentIndex ASC", null);
        }
        /// <summary>
        /// 获取化疗药物
        /// </summary>
        public DataTable GetChemotherapyDrugs()
        {
            SqlHelper context = new SqlHelper();
            return context.ExcuteDatatable("SELECT * FROM dbo.Chemotherapy ORDER BY CurrentIndex ASC", null);
        }
        /// <summary>
        /// 获取遗传风险
        /// </summary>
        public List<GeneticRisksQueryModel> GeneticRisk()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<GeneticRisksQueryModel> GeneticRisksQuery = context.JoinQuery<GeneticRisks, Sys_Organize>((GeneticRisks, org) => new object[] {
                        JoinType.LeftJoin,GeneticRisks.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((GeneticRisks, org) => new GeneticRisksQueryModel
                    {
                        GeneticRisksOID = GeneticRisks.GeneticRisksOID,
                        SampleCode = GeneticRisks.SampleCode,
                        GeneName = GeneticRisks.GeneName,
                        Mutations = GeneticRisks.Mutations,
                        MutationType = GeneticRisks.MutationType,
                        Heterozygosity = GeneticRisks.Heterozygosity,
                        RiskAssessment = GeneticRisks.RiskAssessment,
                        ClassCode = org.ClassCode
                    }).ToList();
                return GeneticRisksQuery;
            }
        }
        /// <summary>
        /// 获取样本质控情况
        /// </summary>
        public List<SampleControlQueryModel> GetSampleQuality()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<SampleControlQueryModel> SampleControlQuery = context.JoinQuery<SampleControl, Sys_Organize>((SampleControl, org) => new object[] {
                        JoinType.LeftJoin,SampleControl.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((SampleControl, org) => new SampleControlQueryModel
                    {
                        SampleControlOID = SampleControl.SampleControlOID,
                        ControlStep = SampleControl.ControlStep,
                        ControlResult = SampleControl.ControlResult,
                        SortNo = SampleControl.SortNo,
                        ClassCode = org.ClassCode
                    }).ToList();
                return SampleControlQuery;
            }
        }
        /// <summary>
        /// 获取靶药参考文献
        /// </summary>
        public string GetTargetDrugReferences()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                try
                {
                    string TargetedMedicineReferencesTxt = context.Query<TargetedMedicineReferences>().Select(a => a.TargetedMedicineReferencesTxt).FirstOrDefault();
                    if (TargetedMedicineReferencesTxt == null)
                    {
                        TargetedMedicineReferencesTxt = "";
                    }
                    return TargetedMedicineReferencesTxt;
                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }
            }
        }
        /// <summary>
        /// 获取基因列表
        /// </summary>
        public List<GeneCorrelationQueryModel> GetGeneList()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<GeneCorrelationQueryModel> GeneCorrelationQuery = context.JoinQuery<GeneCorrelation, CodeValue>((GeneCorrelation, code) => new object[] {
                        JoinType.LeftJoin,GeneCorrelation.Type == code.CodeID
                    }).Select((GeneCorrelation, code) => new GeneCorrelationQueryModel
                    {
                        GeneCorrelationOID = GeneCorrelation.GeneCorrelationOID,
                        GeneName = GeneCorrelation.GeneName,
                        SortNo = GeneCorrelation.SortNo,
                        Type = GeneCorrelation.Type + "",
                        ClassCode = code.ClassCode,
                        Color = code.Reserved3
                    }).ToList();
                return GeneCorrelationQuery;
            }
        }

        /// <summary>
        /// 获取基因类型列表
        /// </summary>
        /// <returns></returns>
        public List<SonCodeTableResponseModel> GetGeneTypeList()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<SonCodeTableResponseModel> AllSonDictionaryDirectory = context.Query<CodeValue>()
                    .Where(a => a.CodeTableID == "Code_GeneType")
                    .Select(a => new SonCodeTableResponseModel
                    {
                        CodeID = a.CodeID,
                        CodeName = a.CodeName,
                        OrderNo = a.OrderNo,
                        Reserved3 = a.Reserved3
                    }).ToList();
                return AllSonDictionaryDirectory;
            }
        }

        /// <summary>
        /// 获取化药参考文献
        /// </summary>
        public string GetChemicalMdicineReferences()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                try
                {
                    string ChemicalDrugReferencesTxt = context.Query<ChemicalDrugReferences>().Select(a => a.ChemicalDrugReferencesTxt).FirstOrDefault();
                    if (ChemicalDrugReferencesTxt == null)
                    {
                        ChemicalDrugReferencesTxt = "";
                    }
                    return ChemicalDrugReferencesTxt;
                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }
            }
        }
    }
}
