﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.GeneDetectionResultModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.GeneDetectionResultSVC
{
    public class GeneDetectionResultSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetGene(GeneDetectionResultRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<DetectionResultQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var GeneQuery = context.JoinQuery<GeneDetectionResult, Sys_Organize>((gene, org) => new object[] {
                        JoinType.LeftJoin,gene.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((gene, org) => new DetectionResultQueryModel
                    {
                        GeneDetectionResultOID = gene.GeneDetectionResultOID,
                        EfficacyProgram = gene.EfficacyProgram,
                        GeneName = gene.GeneName,
                        DetectionSite = gene.DetectionSite,
                        PMID = gene.PMID,
                        TargetedMedicine = gene.TargetedMedicine,
                        MutationSites = gene.MutationSites,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = GeneQuery.Count();

                    List<DetectionResultQueryModel> GeneList = GeneQuery.OrderBy(a => a.EfficacyProgram).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, GeneList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(GeneDetectionResultRequestModel requestModel, LoginInfo user, out LamadaExtention<DetectionResultQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<DetectionResultQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.EfficacyProgram.Contains(requestModel.QueryCondition)
                    || a.GeneName.Contains(requestModel.QueryCondition)
                    || a.DetectionSite.Contains(requestModel.QueryCondition)
                    || a.PMID.Contains(requestModel.QueryCondition)
                    || a.TargetedMedicine.Contains(requestModel.QueryCondition)
                    || a.MutationSites.Contains(requestModel.QueryCondition)
                    );
                }
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveGene(DetectionResultSaveRequestModel gene, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    GeneDetectionResult nowSam = context.Query<GeneDetectionResult>().Where(a => a.GeneDetectionResultOID == gene.GeneDetectionResultOID).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.GeneName = gene.GeneName;
                        nowSam.EfficacyProgram = gene.EfficacyProgram;
                        nowSam.PMID = gene.PMID;
                        nowSam.DetectionSite = gene.DetectionSite;
                        nowSam.TargetedMedicine = gene.TargetedMedicine;
                        nowSam.MutationSites = gene.MutationSites;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new GeneDetectionResult();
                        nowSam.GeneDetectionResultOID = gene.GeneDetectionResultOID;
                        nowSam.GeneName = gene.GeneName;
                        nowSam.EfficacyProgram = gene.EfficacyProgram;
                        nowSam.PMID = gene.PMID;
                        nowSam.DetectionSite = gene.DetectionSite;
                        nowSam.TargetedMedicine = gene.TargetedMedicine;
                        nowSam.MutationSites = gene.MutationSites;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelGene(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<GeneDetectionResult>(a => requestModel.Contains(a.GeneDetectionResultOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportGene(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            using (var context = new MsSqlContext(sqlConn)) {
                context.Session.BeginTransaction();
                try
                {
                    if (file.FileName.Contains(".xlsx"))
                    {
                        workbook = new XSSFWorkbook(file.InputStream);
                    }
                    else
                    {
                        workbook = new HSSFWorkbook(file.InputStream);
                    }
                    sheet = workbook.GetSheetAt(0);
                    int LastRowNum = sheet.LastRowNum;
                    for (int i = 1; i <= LastRowNum; i++)
                    {
                        IRow Irow = sheet.GetRow(i);
                        GeneDetectionResult gene = new GeneDetectionResult();
                        gene.GeneDetectionResultOID = Guid.NewGuid();
                        gene.EfficacyProgram = Irow.GetCell(0).ToString();
                        gene.GeneName = Irow.GetCell(1).ToString();
                        gene.DetectionSite = Irow.GetCell(2).ToString();
                        gene.PMID = Irow.GetCell(3).ToString();
                        gene.TargetedMedicine = Irow.GetCell(4).ToString();
                        gene.MutationSites = Irow.GetCell(5).ToString();
                        InsertSysField(gene, user);
                        context.Insert(gene);

                    }
                    context.Session.CommitTransaction();
                    return true;
                }
                catch (Exception ex)
                {
                    context.Session.RollbackTransaction();
                    throw new Exception(ex.Message);
                }
            }
              
        }
        #endregion
    }
}
