﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.GeneRequestModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.GeneSVC
{
    public class GeneSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetGene(GeneRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<GeneQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var GeneQuery = context.JoinQuery<Gene, Sys_Organize>((gene, org) => new object[] {
                        JoinType.LeftJoin,gene.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((gene, org) => new GeneQueryModel
                    {
                        GeneOID = gene.GeneOID,
                        GeneCode = gene.GeneCode,
                        GeneName = gene.GeneName,
                        GeneRemark = gene.GeneRemark,
                        PMID = gene.PMID,
                        TargetedMedicine = gene.TargetedMedicine,
                        MutationSites = gene.MutationSites,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = GeneQuery.Count();

                    List<GeneQueryModel> GeneList = GeneQuery.OrderBy(a => a.GeneCode).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, GeneList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(GeneRequestModel requestModel, LoginInfo user, out LamadaExtention<GeneQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<GeneQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.GeneCode.Contains(requestModel.QueryCondition)
                    || a.GeneName.Contains(requestModel.QueryCondition)
                    || a.GeneRemark.Contains(requestModel.QueryCondition)
                    || a.PMID.Contains(requestModel.QueryCondition)
                    || a.TargetedMedicine.Contains(requestModel.QueryCondition)
                    || a.MutationSites.Contains(requestModel.QueryCondition)
                    );
                }
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveGene(GeneSaveRequestModel gene, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    Gene nowSam = context.Query<Gene>().Where(a => a.GeneCode == gene.GeneCode).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.GeneName = gene.GeneName;
                        nowSam.GeneCode = gene.GeneCode;
                        nowSam.PMID = gene.PMID;
                        nowSam.GeneRemark = gene.GeneRemark;
                        nowSam.TargetedMedicine = gene.TargetedMedicine;
                        nowSam.MutationSites = gene.MutationSites;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new Gene();
                        nowSam.GeneOID = gene.GeneOID;
                        nowSam.GeneName = gene.GeneName;
                        nowSam.GeneCode = gene.GeneCode;
                        nowSam.PMID = gene.PMID;
                        nowSam.GeneRemark = gene.GeneRemark;
                        nowSam.TargetedMedicine = gene.TargetedMedicine;
                        nowSam.MutationSites = gene.MutationSites;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelGene(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<Gene>(a => requestModel.Contains(a.GeneOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportGene(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    GeneSaveRequestModel gene = new GeneSaveRequestModel();
                    gene.GeneOID = Guid.NewGuid();
                    gene.GeneCode = Irow.GetCell(0).ToString();
                    gene.GeneName = Irow.GetCell(1).ToString();
                    gene.GeneRemark = Irow.GetCell(2).ToString();
                    gene.PMID = Irow.GetCell(3).ToString();
                    gene.TargetedMedicine = Irow.GetCell(4).ToString();
                    gene.MutationSites = Irow.GetCell(5).ToString();
                    DoSaveGene(gene, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
