﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.MISReportModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace G.Buss.BaseInfo.Services.MISReportSVC
{
    public class MISReportSVC :BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetMISReport(MISReportRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<MISReportQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var MISReportQuery = context.JoinQuery<MISReport, Sys_Organize>((MISReport, org) => new object[] {
                        JoinType.LeftJoin,MISReport.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((MISReport, org) => new MISReportQueryModel
                    {
                        MISReportOID = MISReport.MISReportOID,
                        Grade = MISReport.Grade,
                        Scope = MISReport.Scope,
                        Remark = MISReport.Remark,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = MISReportQuery.Count();

                    List<MISReportQueryModel> MISReportList = MISReportQuery.OrderBy(a => a.Grade).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, MISReportList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(MISReportRequestModel requestModel, LoginInfo user, out LamadaExtention<MISReportQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<MISReportQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.Grade.Contains(requestModel.QueryCondition)
                    || a.Scope.Contains(requestModel.QueryCondition)
                    || a.Remark.Contains(requestModel.QueryCondition)
                    );
                }
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveMISReport(MISReportSaveRequestModel MISReport, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    MISReport nowSam = context.Query<MISReport>().Where(a => a.MISReportOID == MISReport.MISReportOID).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.Grade = MISReport.Grade;
                        nowSam.Scope = MISReport.Scope;
                        nowSam.Remark = MISReport.Remark;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new MISReport();
                        nowSam.MISReportOID = MISReport.MISReportOID;
                        nowSam.Grade = MISReport.Grade;
                        nowSam.Scope = MISReport.Scope;
                        nowSam.Remark = MISReport.Remark;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelMISReport(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<MISReport>(a => requestModel.Contains(a.MISReportOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
