﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.PMIDModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.PMIDSVC
{
    public class PMIDSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetPMID(PMIDRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<PMIDQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var PMIDQuery = context.JoinQuery<PMID, Sys_Organize>((pmid, org) => new object[] {
                        JoinType.LeftJoin,pmid.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((pmid, org) => new PMIDQueryModel
                    {
                        PMIDOID = pmid.PMIDOID,
                        Code = pmid.Code,
                        Gene = pmid.Gene,
                        TargetedMedicine = pmid.TargetedMedicine,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = PMIDQuery.Count();

                    List<PMIDQueryModel> PMIDList = PMIDQuery.OrderBy(a => a.Code).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, PMIDList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(PMIDRequestModel requestModel, LoginInfo user, out LamadaExtention<PMIDQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<PMIDQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.Code.Contains(requestModel.QueryCondition)
                    || a.Gene.Contains(requestModel.QueryCondition)
                    || a.TargetedMedicine.Contains(requestModel.QueryCondition)
                    );
                }
               
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSavePMID(PMIDSaveRequestModel pmid, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    PMID nowSam = context.Query<PMID>().Where(a => a.Code == pmid.Code).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.Gene = pmid.Gene;
                        nowSam.Code = pmid.Code;
                        nowSam.TargetedMedicine = pmid.TargetedMedicine;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new PMID();
                        nowSam.PMIDOID = pmid.PMIDOID;
                        nowSam.Gene = pmid.Gene;
                        nowSam.Code = pmid.Code;
                        nowSam.TargetedMedicine = pmid.TargetedMedicine;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelPMID(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<PMID>(a => requestModel.Contains(a.PMIDOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportPMID(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    PMIDSaveRequestModel pmid = new PMIDSaveRequestModel();
                    pmid.PMIDOID = Guid.NewGuid();
                    pmid.Code = Irow.GetCell(0).ToString();
                    pmid.Gene = Irow.GetCell(1).ToString();
                    pmid.TargetedMedicine = Irow.GetCell(2).ToString();
                    DoSavePMID(pmid, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
