﻿using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using G.MES.DataModel;
using Chloe.SqlServer;

namespace G.Buss.BaseInfo.Services.SummarySVC
{
    public class SummarySvc : BaseSvc
    {
        public List<Summary> GetSummaries()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                return context.Query<Summary>().ToList();
            }
        }


        public bool ImportSummary(HttpPostedFile file)
        {
            try
            {
                DataTable table = RenderDataTableFromExcel(file.InputStream, 0, 0);
                List<Summary> summaries = new List<Summary>();
                List<string> SampleNumber = new List<string>();
                Guid PreOID = Guid.Empty;
                for (int i = 0; i < table.Rows.Count; i++)
                {
                    Summary summary = new Summary();
                    Guid OID = Guid.NewGuid();
                    if (string.IsNullOrEmpty(table.Rows[i]["基因"].ToString()))
                    {
                        if (PreOID == Guid.Empty)
                        {
                            throw new Exception("excel第一行数据不能为空");
                        }
                        summary.ParentOID = PreOID;
                        summary.SummaryOID = OID;
                        summary.Gene = table.Rows[i]["基因"].ToString();
                        summary.Position = table.Rows[i]["突变位置及类型"].ToString();
                        summary.Annotation = table.Rows[i]["结果注释"].ToString();
                        summary.Abundance = table.Rows[i]["丰度"].ToString();
                        summary.Mutations = table.Rows[i]["突变说明"].ToString();
                        summary.SampleNumber = table.Rows[i]["样本编号"].ToString();
                        //summary.Sys_CreateBy = user.Sys_UserOID;
                        summary.Sys_Create = DateTime.Now;
                        summary.CurrentIndex = i;
                    }
                    else
                    {
                        summary.ParentOID = Guid.Empty;
                        summary.SummaryOID = OID;
                        summary.Gene = table.Rows[i]["基因"].ToString();
                        summary.Position = table.Rows[i]["突变位置及类型"].ToString();
                        summary.Annotation = table.Rows[i]["结果注释"].ToString();
                        summary.Abundance = table.Rows[i]["丰度"].ToString();
                        summary.Mutations = table.Rows[i]["突变说明"].ToString();
                        summary.SampleNumber = table.Rows[i]["样本编号"].ToString();
                        //summary.Sys_CreateBy = user.Sys_UserOID;
                        summary.Sys_Create = DateTime.Now;
                        summary.CurrentIndex = i;
                        PreOID = OID;
                        SampleNumber.Add(summary.SampleNumber);
                    }
                    summaries.Add(summary);
                }
                using (var context = new MsSqlContext(sqlConn))
                {
                    List<Summary> summarie = context.Query<Summary>().Where(n => SampleNumber.Contains(n.SampleNumber)).ToList();
                    List<Guid> guids = summarie.Select(n => n.SummaryOID).ToList();
                    List<string> numbers = summarie.Select(n => n.SampleNumber).ToList();
                    //删除已存在的数据再重新插入
                    if (summarie.Any())
                    {
                        context.Delete<Summary>(m => numbers.Contains(m.SampleNumber) || guids.Contains(m.ParentOID));
                    }
                    context.InsertRange(summaries);
                }
                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        public static DataTable RenderDataTableFromExcel(Stream ExcelFileStream, int SheetIndex, int HeaderRowIndex)
        {
            XSSFWorkbook workbook = new XSSFWorkbook(ExcelFileStream);
            ISheet sheet = workbook.GetSheetAt(SheetIndex);
            DataTable table = new DataTable();
            IRow headerRow = sheet.GetRow(HeaderRowIndex);
            int cellCount = headerRow.LastCellNum;
            for (int i = headerRow.FirstCellNum; i < cellCount; i++)
            {
                DataColumn column = new DataColumn(headerRow.GetCell(i).StringCellValue);
                table.Columns.Add(column);
            }
            int rowCount = sheet.LastRowNum;
            for (int i = (sheet.FirstRowNum + 1); i <= sheet.LastRowNum; i++)
            {
                IRow row = sheet.GetRow(i);
                DataRow dataRow = table.NewRow();

                for (int j = row.FirstCellNum; j < cellCount; j++)
                {
                    if (row.GetCell(j) != null)
                        dataRow[j] = row.GetCell(j).ToString();
                }

                table.Rows.Add(dataRow);
            }
            ExcelFileStream.Close();
            workbook = null;
            sheet = null;
            return table;
        }
    }
}
