﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.SystemLoggingModel;
using G.MES.DataModel;
using G.MES.ViewModel;

namespace G.Buss.BaseInfo.Services.SystemLoggingAttribute
{
    public class SystenLoggingTableSVC:BaseSvc
    {

        public PageData GetSystemLoggingList(SystemLoggingRequestModel requestModel)
        {
            using (var context = new MsSqlContext(sqlConn))
            {


                int pagesize = requestModel.PageSize;
                int PageNumber = requestModel.PageNumber;
                var where = new LamadaExtention<SystemLoggingModel>();
                if (!string.IsNullOrEmpty(requestModel.RequestUri))
                {
                    where.And(a => a.RequestUri.Contains(requestModel.RequestUri));
                }

                if (!string.IsNullOrEmpty(requestModel.RequestParamaters))
                {
                    where.And(a => a.RequestParamaters.Contains(requestModel.RequestParamaters));
                }

                if (!string.IsNullOrEmpty(requestModel.ResponseParamaters))
                {
                    where.And(a => a.ResponseParamaters.Contains(requestModel.ResponseParamaters));
                }

                if (!string.IsNullOrEmpty(requestModel.UserIP))
                {
                    where.And(a => a.UserIP.Contains(requestModel.UserIP));
                }

                if (!string.IsNullOrEmpty(requestModel.ServerIP))
                {
                    where.And(a => a.ServerIP.Contains(requestModel.ServerIP));
                }

                if (!string.IsNullOrEmpty(requestModel.Browser))
                {
                    where.And(a => a.Browser.Contains(requestModel.Browser));
                }

                if (!string.IsNullOrEmpty(requestModel.UserName))
                {
                    where.And(a => a.UserName.Contains(requestModel.UserName));
                }

                if (requestModel.EnterTime != null)
                {
                    if (!string.IsNullOrEmpty(requestModel.EnterTime.MaxValue))
                    {
                        DateTime MaxValue = Convert.ToDateTime(requestModel.EnterTime.MaxValue);
                        where.And(a => a.EnterTime < MaxValue);
                    }

                    if (!string.IsNullOrEmpty(requestModel.EnterTime.MinValue))
                    {
                        DateTime MinValue = Convert.ToDateTime(requestModel.EnterTime.MinValue);

                        where.And(a => a.EnterTime > MinValue);
                    }
                }

                if (requestModel.EndTime != null)
                {
                    if (!string.IsNullOrEmpty(requestModel.EndTime.MaxValue))
                    {
                        DateTime MaxValue = Convert.ToDateTime(requestModel.EnterTime.MaxValue);

                        where.And(a => a.EndTime < MaxValue);
                    }

                    if (!string.IsNullOrEmpty(requestModel.EndTime.MinValue))
                    {
                        DateTime MinValue = Convert.ToDateTime(requestModel.EnterTime.MinValue);

                        where.And(a => a.EndTime > MinValue);
                    }
                }

                if (requestModel.CostTime != null)
                {
                    if (!string.IsNullOrEmpty(requestModel.CostTime.MaxValue))
                    {
                        int MaxValue = Convert.ToInt32(requestModel.CostTime.MaxValue);

                        where.And(a => a.CostTime < MaxValue);
                    }

                    if (!string.IsNullOrEmpty(requestModel.CostTime.MinValue))
                    {
                        int MinValue = Convert.ToInt32(requestModel.CostTime.MinValue);

                        where.And(a => a.CostTime > MinValue);
                    }
                }

                var queryList = context.JoinQuery<SystemLogging, Sys_User>(
                    (systemLogging, Sys_User) =>
                        new object[]
                        {
                            JoinType.LeftJoin, systemLogging.Sys_User_FK == Sys_User.Sys_UserOID
                        }).Select((SystemLogging, Sys_User) => new SystemLoggingModel
                {
                    SystemLoggingOID = SystemLogging.SysytemLoggingOID,
                    EnterTime = SystemLogging.EnterTime,
                    EndTime = SystemLogging.EndTime,
                    CostTime = SystemLogging.CostTime,
                    RequestUri = SystemLogging.RequestUri,
                    RequestParamaters = "",
                    ResponseParamaters = "",
                    UserIP = SystemLogging.UserIP,
                    ServerIP = SystemLogging.ServerIP,
                    Browser = SystemLogging.Browser,
                    UserName = Sys_User.EmployeeName,
                }).Where(where.GetLambda());
                int totalNum = queryList.Count();
                var systemLoggingList= queryList.OrderByDesc(a => a.EnterTime).TakePage(requestModel.PageNumber, requestModel.PageSize).ToList();
                List<Guid> listGid = systemLoggingList.Select(t => t.SystemLoggingOID).ToList();
                //查找指定分页的
                var findList = context.Query<SystemLogging>().Where(t => listGid.Contains(t.SysytemLoggingOID))
                      .Select(t => new
                      {
                          RequestParamaters = t.RequestParamaters,
                          ResponseParamaters = t.ResponseParamaters,
                          OID=t.SysytemLoggingOID

                      }).ToList();
                foreach (var one in systemLoggingList)
                {
                  
                    one.RequestParamaters = findList.Where(t=>t.OID==one.SystemLoggingOID).FirstOrDefault().RequestParamaters;
                    one.ResponseParamaters = findList.Where(t => t.OID == one.SystemLoggingOID).FirstOrDefault().ResponseParamaters;
                }

                return PageData.Make(requestModel.PageNumber, requestModel.PageSize, totalNum,
                    systemLoggingList);
            }
        }
    }
}
