﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.TMBGradeRequestModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace G.Buss.BaseInfo.Services.TMBGradeSVC
{
    public class TMBGradeSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetTMBGrade(TMBGradeRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<TMBGradeQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var TMBGradeQuery = context.JoinQuery<TMBGrade, Sys_Organize>((TMBGrade, org) => new object[] {
                        JoinType.LeftJoin,TMBGrade.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((TMBGrade, org) => new TMBGradeQueryModel
                    {
                        TMBGradeOID = TMBGrade.TMBGradeOID,
                        Grade = TMBGrade.Grade,
                        Scope = TMBGrade.Scope,
                        Remark = TMBGrade.Remark,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = TMBGradeQuery.Count();

                    List<TMBGradeQueryModel> TMBGradeList = TMBGradeQuery.OrderBy(a => a.Grade).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, TMBGradeList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(TMBGradeRequestModel requestModel, LoginInfo user, out LamadaExtention<TMBGradeQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<TMBGradeQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.Grade.Contains(requestModel.QueryCondition)
                    || a.Scope.Contains(requestModel.QueryCondition)
                    || a.Remark.Contains(requestModel.QueryCondition)
                    );
                }
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveTMBGrade(TMBGradeSaveRequestModel TMBGrade, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    TMBGrade nowSam = context.Query<TMBGrade>().Where(a => a.TMBGradeOID == TMBGrade.TMBGradeOID).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.Grade = TMBGrade.Grade;
                        nowSam.Scope = TMBGrade.Scope;
                        nowSam.Remark = TMBGrade.Remark;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new TMBGrade();
                        nowSam.TMBGradeOID = TMBGrade.TMBGradeOID;
                        nowSam.Grade = TMBGrade.Grade;
                        nowSam.Scope = TMBGrade.Scope;
                        nowSam.Remark = TMBGrade.Remark;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelTMBGrade(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<TMBGrade>(a => requestModel.Contains(a.TMBGradeOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
