﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.TargetedMedicineModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.TargetedMedicineSVC
{
    public class TargetedMedicineSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetTargetedMedicine(TargetedMedicineRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<TargetedMedicineQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var TargetedMedicineQuery = context.JoinQuery<TargetedMedicine, Sys_Organize>((targeted, org) => new object[] {
                        JoinType.LeftJoin,targeted.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((targeted, org) => new TargetedMedicineQueryModel
                    {
                        TargetedMedicineOID = targeted.TargetedMedicineOID,
                        Code = targeted.Code,
                        MedicineName = targeted.MedicineName,
                        DiseaseType = targeted.DiseaseType,
                        Indication = targeted.Indication,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = TargetedMedicineQuery.Count();

                    List<TargetedMedicineQueryModel> TargetedMedicineList = TargetedMedicineQuery.OrderBy(a => a.Code).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, TargetedMedicineList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(TargetedMedicineRequestModel requestModel, LoginInfo user, out LamadaExtention<TargetedMedicineQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<TargetedMedicineQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.Code.Contains(requestModel.QueryCondition)
                    || a.MedicineName.Contains(requestModel.QueryCondition)
                    || a.DiseaseType.Contains(requestModel.QueryCondition)
                    || a.Indication.Contains(requestModel.QueryCondition)
                    );
                }
              
              
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveTargetedMedicine(TargetedMedicineSaveRequestModel targeted, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    TargetedMedicine nowSam = context.Query<TargetedMedicine>().Where(a => a.Code == targeted.Code).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.MedicineName = targeted.MedicineName;
                        nowSam.Code = targeted.Code;
                        nowSam.DiseaseType = targeted.DiseaseType;
                        nowSam.Indication = targeted.Indication;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new TargetedMedicine();
                        nowSam.TargetedMedicineOID = targeted.TargetedMedicineOID;
                        nowSam.MedicineName = targeted.MedicineName;
                        nowSam.Code = targeted.Code;
                        nowSam.DiseaseType = targeted.DiseaseType;
                        nowSam.Indication = targeted.Indication;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelTargetedMedicine(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<TargetedMedicine>(a => requestModel.Contains(a.TargetedMedicineOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportTargetedMedicine(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    TargetedMedicineSaveRequestModel targeted = new TargetedMedicineSaveRequestModel();
                    targeted.TargetedMedicineOID = Guid.NewGuid();
                    targeted.Code = Irow.GetCell(0).ToString();
                    targeted.MedicineName = Irow.GetCell(1).ToString();
                    targeted.DiseaseType = Irow.GetCell(2).ToString();
                    targeted.Indication = Irow.GetCell(3).ToString();
                    DoSaveTargetedMedicine(targeted, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
