﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.Tumorss;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.TumorsTypeSVC
{
    public class TumorsTypeSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetTumorsType(TumorsTypeRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<TumorsTypeQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var TumorsTypeQuery = context.JoinQuery<TumorsType, Sys_Organize>((Tumors, org) => new object[] {
                        JoinType.LeftJoin,Tumors.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((Tumors, org) => new TumorsTypeQueryModel
                    {
                        TumorsTypeOID = Tumors.TumorsTypeOID,
                        TumorsCode = Tumors.TumorsCode,
                        TumorsName = Tumors.TumorsName,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = TumorsTypeQuery.Count();

                    List<TumorsTypeQueryModel> TumorsTypeList = TumorsTypeQuery.OrderBy(a=>a.TumorsCode).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, TumorsTypeList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        public List<TumorsTypeSelectModel> DoGetAllTumorsType(LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    var TumorsTypeQuery = context.JoinQuery<TumorsType, Sys_Organize>((Tumors, org) => new object[] {
                        JoinType.LeftJoin,Tumors.SYS_ORG == org.Sys_OrganizeOID
                    });
                    if (user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                    {
                        TumorsTypeQuery = TumorsTypeQuery.Where((Tumors, org) => org.ClassCode.StartsWith(user.OrgClassCode));
                    }
                    List<TumorsTypeSelectModel> TumorsTypeAll = TumorsTypeQuery.Select((Tumors, org) => new TumorsTypeSelectModel
                    {
                        TumorsCode = Tumors.TumorsCode,
                        TumorsName = Tumors.TumorsName,
                    }).ToList();
                    return TumorsTypeAll;
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(TumorsTypeRequestModel requestModel, LoginInfo user, out LamadaExtention<TumorsTypeQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<TumorsTypeQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.TumorsCode.Contains(requestModel.QueryCondition)|| a.TumorsName.Contains(requestModel.QueryCondition) );
                }
              
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveTumorsType(TumorsTypeSaveRequestModel Tumors, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    TumorsType nowSam = context.Query<TumorsType>().Where(a => a.TumorsCode == Tumors.TumorsCode).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.TumorsName = Tumors.TumorsName;
                        nowSam.TumorsCode = Tumors.TumorsCode;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new TumorsType();
                        nowSam.TumorsTypeOID = Tumors.TumorsTypeOID;
                        nowSam.TumorsName = Tumors.TumorsName;
                        nowSam.TumorsCode = Tumors.TumorsCode;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelTumorsType(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<TumorsType>(a => requestModel.Contains(a.TumorsTypeOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportTumorsType(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    TumorsTypeSaveRequestModel Tumors = new TumorsTypeSaveRequestModel();
                    Tumors.TumorsTypeOID = Guid.NewGuid();
                    Tumors.TumorsCode = Irow.GetCell(0).ToString();
                    Tumors.TumorsName = Irow.GetCell(1).ToString();
                    DoSaveTumorsType(Tumors, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

    }
}
