﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Chloe.SqlServer;
using G.MES.DataModel;
using Common.Unit;
using System.Reflection;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Chloe;
using G.MES.ViewModel;

namespace G.Buss.System
{
    public class RespSvc : BaseSvc
    {

        /// <summary>
        /// /获取责任的列表数据
        /// </summary>
        /// <param name="pageindex"></param>
        /// <param name="pagesize"></param>
        /// <param name="querInfo"></param>
        /// <returns></returns>
        public PageData Resp_GetAllData(int pageindex,int pagesize, ViewModel.QueryRespList querInfo)
        {
            var where = new LamadaExtention<Sys_RESPONSIBILITY>();
            //编码
            if (!string.IsNullOrEmpty(querInfo.RESP_NO))
            {

      
                where.And("RESP_NO", querInfo.RESP_NO, ExpressionType.Contains);
            }
            //名称
            if (!string.IsNullOrEmpty(querInfo.RESP_NAME))
            {
                where.And("RESP_NAME", querInfo.RESP_NAME, ExpressionType.Contains);
            }
            //是否系统责任
            if (querInfo.IsSYS!=null)
            {
                where.And("IsSYS", querInfo.IsSYS, ExpressionType.Equal);
            }
            //是否禁用
            if (querInfo.IsSYS != null)
            {
                where.And("IsDisable", querInfo.IsDisable, ExpressionType.Equal);
            }
            //职责描述
            if (querInfo.RESP_DESC_TEXT != null)
            {
                where.And("RESP_DESC_TEXT", querInfo.RESP_DESC_TEXT, ExpressionType.Contains);
            }
            var queryTotal = context.Query<Sys_RESPONSIBILITY>().Where(where.GetLambda()).Count();
            var query= context.Query<Sys_RESPONSIBILITY>().Where(where.GetLambda()).TakePage(pageindex, pagesize).ToList();

            return PageData.Make(pageindex, pagesize, queryTotal,query);

        }

        public string SaveData(List<ViewModel.Sys_RESPONSIBILITYInfo> list_Resp)
        {
            List<string> oidList = new List<string>();
            using (MsSqlContext context = new MsSqlContext(sqlConn))
            {
                context.Session.BeginTransaction();
                try
                {

                    foreach (var item in list_Resp)
                    {

                        if (string.IsNullOrEmpty(item.viewstate))
                        {
                            var result2 = new {
                                flag = false,
                                data = "",
                                message = "页面修改状态不能为空"
                            };
                            return JsonConvert.SerializeObject(result2);
                        }
                        //插入的
                        if (item.viewstate == ViewModel.Enumerat.viewstate.add.ToString())
                        {
                            Sys_RESPONSIBILITY one = new Sys_RESPONSIBILITY();
                            Guid oid = Guid.NewGuid();
                            one.Sys_RESPONSIBILITYOID = oid;
                            one.RESP_NO = item.RESP_NO;
                            one.RESP_NAME = item.RESP_NAME;
                            one.RESP_DESC_TEXT = item.RESP_DESC_TEXT;
                            one.IsSYS = item.IsSYS;
                            one.IsDisable = item.IsDisable;
                            context.Insert(one);
                            oidList.Add(oid.ToString());
                        }
                        if (item.viewstate == ViewModel.Enumerat.viewstate.modify.ToString())
                        {
                            Guid OID = new Guid(item.Sys_RESPONSIBILITYOID);

                            context.Update<Sys_RESPONSIBILITY>(a => a.Sys_RESPONSIBILITYOID == OID, a => new Sys_RESPONSIBILITY()
                            {
                                RESP_NO = item.RESP_NO,
                                RESP_NAME = item.RESP_NAME,
                                RESP_DESC_TEXT = item.RESP_DESC_TEXT,
                                IsSYS = item.IsSYS,
                                IsDisable = item.IsDisable

                            });
                            oidList.Add(OID.ToString());
                        }

                    }

                    context.Session.CommitTransaction();
                }
                catch
                {
                    if (context.Session.IsInTransaction)
                        context.Session.RollbackTransaction();
                    throw;
                }
            }
            var result = new
            {
                flag = true,
                message = "保存成功",
                data = oidList
            };

            return JsonConvert.SerializeObject(result);
        }
        /// <summary>
        /// 删除列表的数据，
        /// </summary>
        /// <param name="oidlist">OID 串</param>
        public bool Delete(List<string> oidlist)
        {
            bool flag = false;
            try
            {
                List<string> ls = new List<string>();
                foreach (var item in oidlist)
                {
                    Guid OID = new Guid(item);
                    //级联删除
                    context.Delete<Sys_RESPONSIBILITYBPOMAP>(t => t.Sys_RESPONSIBILITY_FK == OID);
                    context.Delete<Sys_POSTN_RESP_MAP>(a => a.Sys_RESPONSIBILITY_FK == OID);

                    context.Delete<Sys_RESPONSIBILITY>(a => a.Sys_RESPONSIBILITYOID == OID);
                }

            }
            catch (Exception e)
            {
                Log4.Write().Error(MethodBase.GetCurrentMethod().Name + e.Message);
                flag = false;
                return flag;
            }

            flag = true;
            return flag;


        }

        /// <summary>
        /// 获取菜单，并且和职责做联动
        /// </summary>
        /// <param name="Sys_RESPONSIBILITY_FK">职责外键</param>
        /// <returns></returns>
        public List<ViewModel.RespMenuTreeInfo> getMenuTree(Guid Sys_RESPONSIBILITY_FK)
        {

       
            var list_menu = context.Query<Sys_BusiPortalMenu>().Where(t=>t.IsStopUsing==false);

            List<ViewModel.RespMenuTreeInfo> list_tree = new List<ViewModel.RespMenuTreeInfo>();
            //得出所有根节点的数据
            int leves = 1;


            var find_parentList = list_menu.Where(t=>t.ParentOID==Guid.Empty).ToList().OrderBy(t => t.SortNo);

            var findzzList = context.Query<Sys_RESPONSIBILITYBPOMAP>().Where(t => t.Sys_RESPONSIBILITY_FK == Sys_RESPONSIBILITY_FK);

            foreach (var item in find_parentList)
            {
                ViewModel.RespMenuTreeInfo v_tree = new ViewModel.RespMenuTreeInfo();
                v_tree.Sys_BusiPortalMenuOID = item.Sys_BusiPortalMenuOID;
                v_tree.name = item.Name;
               
                v_tree.ParentOID = item.ParentOID;
                v_tree.IsSelected = findzzList.Where(t => t.Sys_BusiPortalMenu_FK == item.Sys_BusiPortalMenuOID).Count() == 0 ? false : true;
                v_tree.leves = 1;

                v_tree.children = BuilTree_Resp(list_menu, item.Sys_BusiPortalMenuOID, leves, findzzList);
                list_tree.Add(v_tree);

            }
            return list_tree;

            // return menuSvc.GetAllData(false);
           
        }
        public List<ViewModel.RespMenuTreeInfo> BuilTree_Resp(Chloe.IQuery<Sys_BusiPortalMenu> list_menu, Guid parentOID, int leves,IQuery<Sys_RESPONSIBILITYBPOMAP> findzzList)
        {
            List<ViewModel.RespMenuTreeInfo> list_view_zree = new List<ViewModel.RespMenuTreeInfo>();
           
            var findlist = list_menu.Where(t=>t.ParentOID== parentOID).OrderBy(t => t.SortNo).ToList();

            leves++;
            foreach (var item in findlist)
            {
                ViewModel.RespMenuTreeInfo v_tree = new ViewModel.RespMenuTreeInfo();
                v_tree.Sys_BusiPortalMenuOID = item.Sys_BusiPortalMenuOID;

                v_tree.ParentOID = item.ParentOID;
                v_tree.IsSelected = findzzList.Where(t => t.Sys_BusiPortalMenu_FK == item.Sys_BusiPortalMenuOID).Count() == 0 ? false : true;
                v_tree.leves = leves;
                v_tree.name = item.Name;

                v_tree.children = BuilTree_Resp(list_menu, item.Sys_BusiPortalMenuOID, leves, findzzList);
                list_view_zree.Add(v_tree);
            }
            return list_view_zree;
        }

        /// <summary>
        /// 菜单授权 每次都是先清除了再添加
        /// </summary>
        /// <param name="list_resp"></param>
        /// <returns></returns>
        public dynamic AddMenuRespMap(List<ViewModel.MenuRespMapInfo> list_resp)
        {
            Guid Sys_RESPONSIBILITY_FK = list_resp[0].Sys_RESPONSIBILITY_FK;

           var query = list_resp.GroupBy(x => new { x.Sys_BusiPortalMenu_FK, x.Sys_RESPONSIBILITY_FK }).Select(t=> new {
               Sys_BusiPortalMenu_FK= t.Key.Sys_BusiPortalMenu_FK,
               Sys_RESPONSIBILITY_FK = t.Key.Sys_RESPONSIBILITY_FK,
           }).ToList();
            //删除这个职责下的所有菜单
            context.Delete<Sys_RESPONSIBILITYBPOMAP>(t => t.Sys_RESPONSIBILITY_FK == Sys_RESPONSIBILITY_FK);
            foreach (var item in query)
            {
                Sys_RESPONSIBILITYBPOMAP map = new Sys_RESPONSIBILITYBPOMAP();
                map.Sys_RESPONSIBILITYBPOMAPOID = Guid.NewGuid();
                map.Sys_BusiPortalMenu_FK = item.Sys_BusiPortalMenu_FK ;
                map.Sys_RESPONSIBILITY_FK = item.Sys_RESPONSIBILITY_FK;
                context.Insert(map);

            }
            return new
            {
                flag = true,
                message = "保存成功"
            };
        }

        /// <summary>
        /// 菜单授权 重置
        /// </summary>
        /// <param name="list_resp"></param>
        /// <returns></returns>
        public dynamic ResetMenuRespMap(string Sys_RESPONSIBILITY_FK)
        {
            Guid Sys_RESPONSIBILITY_FK_guid = new Guid(Sys_RESPONSIBILITY_FK);
            //删除这个职责下的所有菜单
            context.Delete<Sys_RESPONSIBILITYBPOMAP>(t => t.Sys_RESPONSIBILITY_FK == Sys_RESPONSIBILITY_FK_guid);

            return new
            {
                flag = true,
                message = "重置成功"
            };
           
        }


        /// <summary>
        /// 获取责任下的岗位
        /// </summary>
        /// <returns></returns>
        public  PageData  GetRespPostList(int pageindex,int pagesize, Guid Sys_RESPONSIBILITY_FK,  ViewModel.RespPostInfo queryModel)
        {
            var where = new LamadaExtention<ViewModel.RespPostInfo>();
            if (!string.IsNullOrEmpty(queryModel.postID))
            {
                where.And("postID", queryModel.postID, ExpressionType.Contains);
            }
            if (!string.IsNullOrEmpty(queryModel.postName))
            {
                where.And("postName", queryModel.postName, ExpressionType.Contains);
            }
            if (!string.IsNullOrEmpty(queryModel.divisionName))
            {
                where.And("divisionName", queryModel.divisionName, ExpressionType.Contains);
            }
            if (!string.IsNullOrEmpty(queryModel.orgName))
            {
                where.And("orgName", queryModel.orgName, ExpressionType.Contains);
            }

            where.And("Sys_RESPONSIBILITY_FK", Sys_RESPONSIBILITY_FK, ExpressionType.Equal);

            var query = context.JoinQuery<Sys_POSTN_RESP_MAP, Sys_Post, Sys_Organize, Sys_Organize>((postrespt, post, bm, gs) => new object[]
               {
                JoinType.LeftJoin, postrespt.Sys_Post_FK == post.Sys_PostOID ,         /* 表 User 和 City 进行Left连接 */
                JoinType.LeftJoin, post.Sys_DivisionOID == bm.Sys_OrganizeOID ,  /* 表 City 和 Province 进行Left连接 */
                JoinType.LeftJoin, post.Sys_Organize_FK == gs.Sys_OrganizeOID
               })
             .Select((postrespt, post, bm, gs) => new ViewModel.RespPostInfo
             {
                 Sys_POSTN_RESP_MAPOID = postrespt.Sys_POSTN_RESP_MAPOID,
                 postName = post.PostName,
                 postID = post.PostID,
                 divisionName = bm.OrgName,
                 orgName = bm.OrgName,
                 Sys_RESPONSIBILITY_FK = postrespt.Sys_RESPONSIBILITY_FK

             })  /* 投影成匿名对象 */
             .Where(where.GetLambda());     /* 进行条件过滤 */
             

          

            return PageData.Make(pageindex, pagesize, query.Count(), query.TakePage(pageindex,pagesize).ToList());

             

        }

        /// <summary>
        /// 添加职责下的岗位
        /// </summary>
        /// <param name="lists"></param>
        /// <returns></returns>
        public PageReturnInfo AddRespPost(List<Sys_POSTN_RESP_MAP> lists)
        {
            PageReturnInfo pageReturnInfo = new PageReturnInfo();

            Guid Sys_RESPONSIBILITY_FK = lists[0].Sys_RESPONSIBILITY_FK;//获取职责的外键
            if (context.Query<Sys_RESPONSIBILITY>().Where(t => t.Sys_RESPONSIBILITYOID == Sys_RESPONSIBILITY_FK).Count() == 0)
            {
                pageReturnInfo.flag = false;
                pageReturnInfo.messages = "责任已经被删除，请关闭所有页面再试";
                return pageReturnInfo;

            }
            var query_map = context.Query<Sys_POSTN_RESP_MAP>().Where(t => t.Sys_RESPONSIBILITY_FK == Sys_RESPONSIBILITY_FK).ToList();
            using (MsSqlContext context = new MsSqlContext(sqlConn))
            {
                context.Session.BeginTransaction();
                try
                {
                    /* do some things here */
                    foreach (var item in lists)
                    {
                        //接口判断不能重复插入
                        if (query_map.Where(t => t.Sys_Post_FK == item.Sys_Post_FK).Count() == 0)
                        {
                            Sys_POSTN_RESP_MAP map = new Sys_POSTN_RESP_MAP();
                            map.Sys_POSTN_RESP_MAPOID = Guid.NewGuid();
                            map.Sys_Post_FK = item.Sys_Post_FK;
                            map.Sys_RESPONSIBILITY_FK = Sys_RESPONSIBILITY_FK;
                            context.Insert(map);
                        }
                       

                    }
                    context.Session.CommitTransaction();
                }
                catch(Exception e)
                {
                    if (context.Session.IsInTransaction)
                        context.Session.RollbackTransaction();
                    Log4.Write().Error(MethodBase.GetCurrentMethod().Name + e.Message);
                    throw;
                }
            }
            pageReturnInfo.flag = true;
            pageReturnInfo.messages = "保存成功";
            return pageReturnInfo;

        }


        /// <summary>
        /// 职责下岗位删除，支持多选
        /// </summary>
        /// <param name="oidlist"></param>
        /// <returns></returns>
        public bool DelRespPost(List<string> oidlist)
        {
            bool flag = false;
            using (MsSqlContext context = new MsSqlContext(sqlConn))
            {
                context.Session.BeginTransaction();
                try
                {
                    foreach (var item in oidlist)
                    {
                        Guid OID = new Guid(item);
                        context.Delete<Sys_POSTN_RESP_MAP>(t => t.Sys_POSTN_RESP_MAPOID == OID);
                    }

                    context.Session.CommitTransaction();
                }
                catch
                {
                    if (context.Session.IsInTransaction)
                        context.Session.RollbackTransaction();
                    throw;
                }
            }
            flag = true;
            return flag;

        }



    }
}
