﻿using G.Buss.BaseInfo.Services.ExportWordSVC;
using G.Buss.BaseInfo.ViewModel.ClientInfoModel;
using G.Buss.BaseInfo.ViewModel.CodeTableModel;
using G.Buss.BaseInfo.ViewModel.GeneCorrelationModel;
using G.Buss.BaseInfo.ViewModel.MISReportModel;
using G.Buss.BaseInfo.ViewModel.SampleControlModel;
using G.Buss.BaseInfo.ViewModel.TMBGradeRequestModel;
using G.MES.DataModel;
using Spire.Doc;
using Spire.Doc.Documents;
using Spire.Doc.Fields;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WordByDocx.Word
{
    public class ReplaceText
    {
        private Spire.Doc.Document document;

        public ReplaceText(Spire.Doc.Document document)
        {
            this.document = document;
        }
        /// <summary>
        /// 替换受试者基本信息
        /// </summary>
        public void ReplaceUserInfo(Guid guid)
        {
            ClientInfoDetailModel clientInfo = GetData.GetUserInfo(guid);
            document.ReplaceFunc("#{Name}", clientInfo.Name);
            document.ReplaceFunc("#{Sex}", clientInfo.Sex == 1 ? "男" : "女");
            document.ReplaceFunc("#{Birthday}", clientInfo.Birth.ToLocalLongTime());
            document.ReplaceFunc("#{IdCard}", clientInfo.IDCard);
            document.ReplaceFunc("#{Phone}", clientInfo.Phone);
            document.ReplaceFunc("#{SampleCode}", clientInfo.SampleCode);
            document.ReplaceFunc("#{SampleType}", clientInfo.SampleType);
            document.ReplaceFunc("#{SampleDetection}", clientInfo.SampleDetection);
            document.ReplaceFunc("#{SamplingTime}", clientInfo.SamplingTime.ToLocalLongTime());
            document.ReplaceFunc("#{TechnologyDetection}", clientInfo.TechnologyDetection);
            document.ReplaceFunc("#{TumorsType}", clientInfo.TumorsType);
            document.ReplaceFunc("#{TNM}", clientInfo.TNM);
            document.ReplaceFunc("#{IsEssential}", clientInfo.IsEssential == true ? "是" : "否");
            document.ReplaceFunc("#{ClinicalDiagnosis}", clientInfo.ClinicalDiagnosis);
            document.ReplaceFunc("#{FamilyHistory}", clientInfo.FamilyHistory);
            document.ReplaceFunc("#{PersonalHistory}", clientInfo.PersonalHistory);
        }

        /// <summary>
        /// 替换检测结果
        /// </summary>
        public void ReplaceSummary()
        {
            DataTable summaries = GetData.GetSummary();
            string[] heards = new string[] { "基因突变有对应药物提示", "突变位置及类型", "结果注释", "丰度" };
            string[] tableHeards = new string[] { "Gene", "Position", "Annotation", "Abundance" };
            Section section = document.Sections[0];
            TextSelection selection = document.FindString("#{SummaryTable}", true, true);
            TextRange range = selection.GetAsOneRange();
            Paragraph paragraph = range.OwnerParagraph;
            Body body = paragraph.OwnerTextBody;
            int index = body.ChildObjects.IndexOf(paragraph);
            Table table = section.AddTable(true);
            table.ResetCells(summaries.Rows.Count + 1, heards.Length);
            //填充数据与样式
            for (int i = 0; i < heards.Length; i++)
            {
                TableRow Frow = table.Rows[0];
                Frow.IsHeader = true;
                Frow.Height = 30;
                Frow.HeightType = TableRowHeightType.Exactly;
                Frow.RowFormat.BackColor = Color.LightGoldenrodYellow;
                Frow.Cells[i].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p3 = Frow.Cells[i].AddParagraph();
                p3.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p3.AppendText(heards[i]);
            }
            for (int r = 0; r < summaries.Rows.Count; r++)
            {
                TableRow dataRow = table.Rows[r + 1];
                dataRow.Height = 25;
                for (int i = 0; i < tableHeards.Length; i++)
                {
                    dataRow.Cells[i].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                    Paragraph p = dataRow.Cells[i].AddParagraph();
                    p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                    string val = summaries.Rows[r][tableHeards[i]].ToString();
                    p.AppendText(val);
                    if (string.IsNullOrEmpty(val))
                    {
                        table.ApplyVerticalMerge(i, r, r + 1);
                    }
                    //else
                    //{
                    //    preOID = summaries.Rows[r]["SummaryOID"].ToString();
                    //}
                }
            }
            body.ChildObjects.Remove(paragraph);
            body.ChildObjects.Insert(index, table);
        }

        /// <summary>
        /// 替换化疗结果
        /// </summary>
        public void ReplaceChemotherapy()
        {
            DataTable summaries = GetData.GetChemotherapy();
            string[] heards = new string[] { "药物", "基因", "检测位点", "基因型", "用药提示", "级别" };
            string[] tableHeards = new string[] { "DrugPlan", "Gene", "TestSite", "Genotype", "MedicationPrompts", "Levels" };
            Section section = document.Sections[0];
            TextSelection selection = document.FindString("#{Chemotherapys}", true, true);
            TextRange range = selection.GetAsOneRange();
            Paragraph paragraph = range.OwnerParagraph;
            Body body = paragraph.OwnerTextBody;
            int index = body.ChildObjects.IndexOf(paragraph);
            Table table = section.AddTable(true);
            table.ResetCells(summaries.Rows.Count + 1, heards.Length);
            //填充数据与样式
            for (int i = 0; i < heards.Length; i++)
            {
                TableRow Frow = table.Rows[0];
                Frow.IsHeader = true;
                Frow.Height = 30;
                Frow.HeightType = TableRowHeightType.Exactly;
                Frow.RowFormat.BackColor = Color.LightGoldenrodYellow;
                Frow.Cells[i].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p3 = Frow.Cells[i].AddParagraph();
                p3.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p3.AppendText(heards[i]);
            }
            for (int r = 0; r < summaries.Rows.Count; r++)
            {
                TableRow dataRow = table.Rows[r + 1];
                dataRow.Height = 25;
                for (int i = 0; i < tableHeards.Length; i++)
                {
                    dataRow.Cells[i].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                    Paragraph p = dataRow.Cells[i].AddParagraph();
                    p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                    string val = summaries.Rows[r][tableHeards[i]].ToString();
                    p.AppendText(val);
                    if (string.IsNullOrEmpty(val))
                    {
                        table.ApplyVerticalMerge(i, r, r + 1);
                    }
                }
            }
            body.ChildObjects.Remove(paragraph);
            body.ChildObjects.Insert(index, table);
        }
        /// <summary>
        /// 替换TMB表格
        /// </summary>
        public void ReplaceTMBTable()
        {
            List<TMBTable> tMBs = GetData.GetTMBTable();
            Section section = document.Sections[0];
            TextSelection selection = document.FindString("#{TMBTable}", true, true);
            TextRange range = selection.GetAsOneRange();
            Paragraph paragraph = range.OwnerParagraph;
            Body body = paragraph.OwnerTextBody;
            int index = body.ChildObjects.IndexOf(paragraph);
            Table table = section.AddTable(true);
            table.ResetCells(tMBs.Count + 1, 3);
            table.ApplyHorizontalMerge(0, 0, 2);
            //填充数据与样式
            TableRow Frow = table.Rows[0];
            Frow.IsHeader = true;
            Frow.Height = 30;
            Frow.HeightType = TableRowHeightType.Exactly;
            Frow.RowFormat.BackColor = Color.LimeGreen;
            Frow.Cells[0].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
            Paragraph p3 = Frow.Cells[0].AddParagraph();
            p3.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
            p3.AppendText("TMB分为三个等级");
            for (int r = 0; r < tMBs.Count; r++)
            {
                TableRow dataRow = table.Rows[r + 1];
                dataRow.Height = 25;
                //范围
                dataRow.Cells[0].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p = dataRow.Cells[0].AddParagraph();
                p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p.AppendText(tMBs[r].Grade);
                //等级
                dataRow.Cells[1].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p1 = dataRow.Cells[1].AddParagraph();
                p1.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p1.AppendText(tMBs[r].Scope);
                //说明
                dataRow.Cells[2].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p2 = dataRow.Cells[2].AddParagraph();
                p2.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p2.AppendText(tMBs[r].Remark);
            }
            body.ChildObjects.Remove(paragraph);
            body.ChildObjects.Insert(index, table);
        }

        /// <summary>
        /// 替换MSI表格
        /// </summary>
        public void ReplaceMSITable()
        {
            List<MISReportQueryModel> MSIS = GetData.GetMSITable();
            Section section = document.Sections[0];
            TextSelection selection = document.FindString("#{MSITable}", true, true);
            TextRange range = selection.GetAsOneRange();
            Paragraph paragraph = range.OwnerParagraph;
            Body body = paragraph.OwnerTextBody;
            int index = body.ChildObjects.IndexOf(paragraph);
            Table table = section.AddTable(true);
            table.ResetCells(MSIS.Count + 1, 3);
            table.ApplyHorizontalMerge(0, 0, 2);
            //填充数据与样式
            TableRow Frow = table.Rows[0];
            Frow.IsHeader = true;
            Frow.Height = 30;
            Frow.HeightType = TableRowHeightType.Exactly;
            Frow.RowFormat.BackColor = Color.LimeGreen;
            Frow.Cells[0].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
            Paragraph p3 = Frow.Cells[0].AddParagraph();
            p3.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
            p3.AppendText("MSI分为三个等级");
            for (int r = 0; r < MSIS.Count; r++)
            {
                TableRow dataRow = table.Rows[r + 1];
                dataRow.Height = 25;
                //范围
                dataRow.Cells[0].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p = dataRow.Cells[0].AddParagraph();
                p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p.AppendText(MSIS[r].Grade);
                //等级
                dataRow.Cells[1].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p1 = dataRow.Cells[1].AddParagraph();
                p1.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p1.AppendText(MSIS[r].Scope);
                //说明
                dataRow.Cells[2].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p2 = dataRow.Cells[2].AddParagraph();
                p2.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p2.AppendText(MSIS[r].Remark);
            }
            body.ChildObjects.Remove(paragraph);
            body.ChildObjects.Insert(index, table);
        }

        /// <summary>
        /// 替换基因列表
        /// </summary>
        public void ReplaceGeneList()
        {
            List<GeneCorrelationQueryModel> GeneList = GetData.GetGeneList();
            Section section = document.Sections[0];
            TextSelection selection = document.FindString("#{GeneList}", true, true);
            TextRange range = selection.GetAsOneRange();
            Paragraph paragraph = range.OwnerParagraph;
            Body body = paragraph.OwnerTextBody;
            int index = body.ChildObjects.IndexOf(paragraph);
            Table table = section.AddTable(true);
            double divisor = 6.00;
            double ce = GeneList.Count / divisor;
            int tableRows = (int)Math.Ceiling(ce);
            table.ResetCells(tableRows, 6);
            for (int r = 0; r < tableRows; r++)
            {
                TableRow dataRow = table.Rows[r];
                dataRow.Height = 10;
                for (int i = 0; i < 6; i++)
                {
                    int currentIndex = 6 * r + i;
                    GeneCorrelationQueryModel gene = null;
                    if (currentIndex >= GeneList.Count)
                    {
                        break;
                    }
                    else
                    {
                        gene = GeneList[currentIndex];
                        dataRow.Cells[i].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                        dataRow.Cells[i].CellFormat.BackColor = System.Drawing.ColorTranslator.FromHtml(gene.Color);
                        Paragraph p = dataRow.Cells[i].AddParagraph();
                        p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                        TextRange bg = p.AppendText(gene.GeneName);
                        bg.CharacterFormat.FontSize = 8;
                    }
                }
            }
            body.ChildObjects.Remove(paragraph);
            body.ChildObjects.Insert(index, table);

            List<SonCodeTableResponseModel> GeneTypeList = GetData.GetGeneTypeList();
            Section section2 = document.Sections[0];
            TextSelection selection2 = document.FindString("#{GeneTypeList}", true, true);
            TextRange range2 = selection2.GetAsOneRange();
            Paragraph paragraph2 = range2.OwnerParagraph;
            Body body2 = paragraph2.OwnerTextBody;
            int index2 = body2.ChildObjects.IndexOf(paragraph2);
            Table table2 = section2.AddTable(true);
            table2.ResetCells(GeneTypeList.Count, 2);
            for (int i = 0; i < GeneTypeList.Count; i++)
            {
                SonCodeTableResponseModel GeneType = GeneTypeList[i];
                TableRow dataRow = table2.Rows[i];
                dataRow.Cells[0].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                dataRow.Cells[0].CellFormat.BackColor = System.Drawing.ColorTranslator.FromHtml(GeneType.Reserved3);
                Paragraph p2 = dataRow.Cells[0].AddParagraph();
                p2.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;

                dataRow.Cells[1].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p = dataRow.Cells[1].AddParagraph();
                p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                TextRange bg = p.AppendText(GeneType.CodeName);
                bg.CharacterFormat.FontSize = 10;
            }
            body2.ChildObjects.Remove(paragraph2);
            body2.ChildObjects.Insert(index2, table2);
        }

        /// <summary>
        /// 替换样本质控情况
        /// </summary>
        public void ReplaceSampleQuality()
        {
            List<SampleControlQueryModel> Sample = GetData.GetSampleQuality();
            string[] header = new string[] { "质控环节", "质控结果" };
            Section section = document.Sections[0];
            TextSelection selection = document.FindString("#{SampleQuality}", true, true);
            TextRange range = selection.GetAsOneRange();
            Paragraph paragraph = range.OwnerParagraph;
            Body body = paragraph.OwnerTextBody;
            int index = body.ChildObjects.IndexOf(paragraph);
            Table table = section.AddTable(true);
            table.ResetCells(Sample.Count + 1, 2);
            //填充数据与样式
            TableRow Frow = table.Rows[0];
            for (int i = 0; i < header.Length; i++)
            {
                Frow.IsHeader = true;
                Frow.Height = 30;
                Frow.HeightType = TableRowHeightType.Exactly;
                Frow.RowFormat.BackColor = Color.LimeGreen;
                Frow.Cells[i].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p3 = Frow.Cells[i].AddParagraph();
                p3.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p3.AppendText(header[i]);
            }
            for (int r = 0; r < Sample.Count; r++)
            {
                TableRow dataRow = table.Rows[r + 1];
                dataRow.Height = 25;
                //质控步骤
                dataRow.Cells[0].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p = dataRow.Cells[0].AddParagraph();
                p.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p.AppendText(Sample[r].ControlStep);
                //质控结果
                dataRow.Cells[1].CellFormat.VerticalAlignment = Spire.Doc.Documents.VerticalAlignment.Middle;
                Paragraph p1 = dataRow.Cells[1].AddParagraph();
                p1.Format.HorizontalAlignment = Spire.Doc.Documents.HorizontalAlignment.Center;
                p1.AppendText(Sample[r].ControlResult);
            }
            body.ChildObjects.Remove(paragraph);
            body.ChildObjects.Insert(index, table);
        }



        /// <summary>
        /// 替换靶药文献
        /// </summary>
        public void ReplaceTargetDrugReferences()
        {
            string TargetDrugReferences = GetData.GetTargetDrugReferences();
            TargetDrugReferences = TargetDrugReferences.Replace("<p>", "");
            TargetDrugReferences = TargetDrugReferences.Replace("</p>", "");
            document.ReplaceFunc("#{TargetDrugReferences}", TargetDrugReferences);
        }

        /// <summary>
        /// 替换化药文献
        /// </summary>
        public void ReplaceChemicalMedicineReferences()
        {
            string ChemicalMedicineReferences = GetData.GetChemicalMedicineReferences();
            ChemicalMedicineReferences = ChemicalMedicineReferences.Replace("<p>", "");
            ChemicalMedicineReferences = ChemicalMedicineReferences.Replace("</p>", "");
            document.ReplaceFunc("#{ChemicalMedicineReferences}", ChemicalMedicineReferences);
        }
    }
}
